/*
PFEdemo1 - Probas

Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import simu.*;

//utilise Proba et MoreProba

/**
 * Algorithme de simulation pour SimuLab.
 * DrawGamma
 */

//	compilation:
// javac -target 1.1 -bootclasspath ~/Java/jdk1.1/JDKClasses.zip -extdirs "" -encoding ISO-8859-1 -classpath .:SimuLab.jar DrawGamma.java

public class DrawGamma extends SimuApplet {
	final static int numax = 10;
	final static int tempo = 1000; // un affichage / 1s
	final static int npts = 128;
	boolean tcl = true;
	Affichage affPlotLive;
	String demo;
	long ct;
	int nt;
	double[] tx;
	double[] tft;
	double[] tfg;

    public DrawGamma() {
		
        // paramtres d'entre
        ParamIn demoParam = new ParamIn();
        demoParam.type = "string";
        demoParam.label = "demo";
        demoParam.titre = "demo";
        demoParam.acquisition = "choix";
		String[] lchoix = {"F(t)/f(t)", "f(t)/lambda"};
		demoParam.choix = lchoix;
        demoParam.defaut = "F(t)/f(t)";
        addParamIn(demoParam);
        
        ParamIn tclParam = new ParamIn();
        tclParam.type = "string";
        tclParam.label = "tcl";
        tclParam.titre = "Gauss limit";
        tclParam.acquisition = "case";
        tclParam.defaut = "on";
        addParamIn(tclParam);
        
		
        // paramtres de sortie
		
        ParamOut x = new ParamOut();
        x.type = "nombre";
        x.label = "x";
        x.titre = "";
        addParamOut(x);
        ParamOut dy = new ParamOut();
        dy.type = "nombre";
        dy.label = "dy";
        dy.titre = "";
        addParamOut(dy);
		
		// Gauss limit
        ParamOut gy = new ParamOut();
        gy.type = "nombre";
        gy.label  = "gy";
        gy.titre = "";
        addParamOut(gy);
        
		// infos (n itrations)
        ParamOut info = new ParamOut();
        info.type = "string";
        info.label = "info";
        addParamOut(info);
		
        // options d'affichage
		appletDoc("DrawGamma_doc.xml");
        
        // affichage "plotlive"
        affPlotLive = new Affichage();
        affPlotLive.type = "plotlive";
        affPlotLive.titre = "Lois Gamma pour nu=1..10";
        String[][] pparams = {{"x", "dy"},{"x","gy"}};
        affPlotLive.params = pparams;
		affPlotLive.points = "none";
		//affPlotLive.effacer = true;
		affPlotLive.connecter = true;
        addAffichage(affPlotLive);
    }
	
    // calcul live (utilis avec affichage "plotlive")
    // initialisation
    public void initCalculLive(ListeValeurs in) throws SimuException {
		
        demo = in.lireString("demo");
		tcl = "on".equals(in.lireString("tcl"));
		
		double[] boxF = MoreProba.bestBox2(1.0, 5);
		double xmin =0;
		double xmax = boxF[1];
		double ymin = boxF[2];
		double ymax = boxF[3];
		
		affPlotLive.xdebut = "" + xmin;
		affPlotLive.xfin = "" + xmax;
		affPlotLive.ydebut = "" + ymin;
		affPlotLive.yfin = "" + ymax;
		
		// Compute reduced normal f.r.
		tx = new double[npts];
		tft = new double[npts];
		
		for (int i=0; i<npts; i++)
			tx[i] = xmin + i*(xmax-xmin)/(npts-1.0);
		
		if (tcl) {
			if ("F(t)/f(t)".equals(demo))
				tfg = Proba.fnormal(numax, Math.sqrt(numax), tx);
			else
				tfg = Proba.pnormal(numax, Math.sqrt(numax), tx);
		}
		
		nt = 0;
		ct = System.currentTimeMillis();
    }
    
    // Calcul du temps t au temps t+1.
    public ListeValeurs calculLive() throws SimuException {
        if (nt >= numax)
            return null; // fin du calcul
		
		nt++;
		
		double nu = nt;
		
        ListeValeurs out = new ListeValeurs();
		
		if ("F(t)/f(t)".equals(demo)) {
			tft = Proba.fgamma(nu, 1.0, tx);
			for (int i=0; i<npts; i++) {
				out.ajouterDouble("x", tx[i]);
				out.ajouterDouble("dy", tft[i]);
			}
		} else {
			// Draw p.d.f.
			try {
				tft = Proba.pgamma(nu, 1.0, tx);
				for (int i=0; i<npts; i++) {
					out.ajouterDouble("x", tx[i]);
					out.ajouterDouble("dy", tft[i]);
				}
			} catch (Exception ex) {
				System.err.println(ex.getClass().getName() + ": " + ex.getMessage());
			}
		}
		if (tcl) {
			for (int i=0; i<npts; i++)
				out.ajouterDouble("gy", tfg[i]);
		} else { // pb affichage: comment ne pas avoir  mettre de valeur ici ?
			out.ajouterDouble("gy", 0);
		}
		
		out.ajouterString("fin", "courbe");
		
		out.ajouterString("info", demo + "    nu = " + nu);
		
		// pour bloquer le programme pendant dt-tempo secondes
		long ct2 = System.currentTimeMillis();
		if (ct2 - ct < tempo) {
	        try {
	            Thread.currentThread().sleep(tempo - (ct2 - ct));
	        } catch (InterruptedException e) {}
			ct += tempo;
		} else
			ct = ct2;
		
        return out;
    }
}
