/*
Flux2D2 - algorithme de simulation pour SimuApplet

Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import simu.*;

/**
 * Algorithme de simulation pour SimuApplet.
 * Flux2D2.
 */
public class Flux2D2 extends SimuApplet {
	
	final static int npts = 128;
	final static double mu = 3.0; // per pixel
	final static double lambda = 0.001;
	final static int np = 128;
	final static int tmax = 100000;
	int nipa;
	double[] tr = {0.,1.,0.,0.,0.,1.};
    double ta[][];
    double ts[][];
	double moyenne;
	//double zmin, zmax;
	long ct;
	int nt;

    public Flux2D2(){
        
        // paramtres d'entre
        ParamIn nipaParam = new ParamIn();
        nipaParam.type = "nombre";
        nipaParam.label = "nipa";
        nipaParam.titre = "itrations/aff";
        nipaParam.acquisition = "champ";
        nipaParam.defaut = "10";
        addParamIn(nipaParam);
		
        // paramtres de sortie
        ParamOut tsp = new ParamOut();
        tsp.type = "tableau";
        tsp.label = "ts";
        addParamOut(tsp);

        // options d'affichage
        Affichage p = new Affichage();
        p.type = "image";
        p.titre = "Flux 2D";
        String[][]pparams = {{"ts"}};
        p.params = pparams;
        p.imgdimx = np;
        p.imgdimy = np;
        addAffichage(p);

}


// calcul en image (utilis avec affichage image)
    public void initCalculLive(ListeValeurs in) throws SimuException {
		
		nipa = in.lireEntier("nipa");
		
        ta = new double[np][np];
        ts = new double[np][np];
		
		//zmax = 0;
		//zmin = mu + 2.0*Math.sqrt(mu);
		// Generate the Poisson sample size.
		int[] num = Proba.sPoisson(lambda*tmax, 1);
		tr[1] = 1.0/np;
		tr[5] = tr[1];
		tr[0] = -tr[1]/2.;
		tr[3] = tr[0];
		//System.out.println("tr[0]="+tr[0]+"  tr[1]="+tr[1]);
		//System.out.println("tr[3]="+tr[3]+"  tr[5]="+tr[5]);
		
        for(int i=0; i<np; i++)
            for(int j=0; j<np; j++)
				ta[i][j] = 0;
		moyenne = 0;
		
		nt = 0;
		ct = System.currentTimeMillis();
    }
    
    public ListeValeurs calculLive() throws SimuException {
        if (nt >= tmax)
            return null; // fin du calcul
		
		for (int ii=0; ii<nipa; ii++) {
			nt++;
			
			int i2 = (int)Math.round((Math.random()-tr[0])/tr[1]) - 1;
			int j2 = (int)Math.round((Math.random()-tr[3])/tr[5]) - 1;
			//System.out.println("i2="+i2+"  j2="+j2);
			ta[i2][j2] += 1;
			moyenne += 1.0/(np*np);
		}
		double v;
		double moy2 = 128/moyenne;
        for(int i=0; i<np; i++)
            for(int j=0; j<np; j++) {
				v = ta[i][j] * moy2;
				if (v > 255)
					v = 255;
				ts[i][j] = v;
			}
		
        ListeValeurs out = new ListeValeurs();
        out.ajouter("ts", ts);
		
        return out;
    }
    
    public ListeValeurs calcul(ListeValeurs in) throws SimuException {
        throw new SimuException("calcul n'est pas implment dans Flux2D2: " +
            "utiliser calculLive  la place");
    }

}
