/*
PFEdemo1 - Probas

Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import simu.*;

//utilise Proba et MoreProba

/**
 * Algorithme de simulation pour SimuLab.
 * Histogram
 */

//	compilation: javac -target 1.1 -bootclasspath ~/Java/jdk1.1/JDKClasses.zip -extdirs "" -encoding ISO-8859-1 -classpath .:SimuLab.jar Histogram.java

public class Histogram extends SimuApplet {
	Affichage affPlot;
	double mu;
	int npts;
	// variable/constantes pour l'affichage "plotlive":
	final static int NTIMES = 400;
	final static int tempo = 500; // un affichage / 0.5s
	final static int nGauss = 128;
	final static int nipa = 1; // nombre d'itrations par affichage
	Affichage affPlotLive;
	long ct;
	int nt;
	double[] histoM;
	double hmin, hmax;
	int nh;
	boolean distrib = true;
	double tfx[];
	double tfy[];
	boolean tcl = true;
	double[] tx;
	double[] gauss;

    public Histogram() {
        // paramtres d'entre
        ParamIn muParam = new ParamIn();
        muParam.type = "nombre";
        muParam.label = "mu";
        muParam.titre = "";
        muParam.acquisition = "champ";
        muParam.defaut = "5";
        addParamIn(muParam);
        
        ParamIn nptsParam = new ParamIn();
        nptsParam.type = "nombre";
        nptsParam.label = "npts";
        nptsParam.titre = "sample size";
        nptsParam.acquisition = "champ";
        nptsParam.defaut = "30";
        addParamIn(nptsParam);
        
        ParamIn distribParam = new ParamIn();
        distribParam.type = "string";
        distribParam.label = "distrib";
        distribParam.titre = "distribution";
        distribParam.acquisition = "case";
        distribParam.defaut = "on";
        addParamIn(distribParam);
        
        ParamIn tclParam = new ParamIn();
        tclParam.type = "string";
        tclParam.label = "tcl";
        tclParam.titre = "Gauss limit";
        tclParam.acquisition = "case";
        tclParam.defaut = "on";
        addParamIn(tclParam);
        
        // paramtres de sortie
		
		// histogramme
        ParamOut hx = new ParamOut();
        hx.type = "nombre";
        hx.label = "hx";
        hx.titre = "";
        addParamOut(hx);
        ParamOut hy = new ParamOut();
        hy.type = "nombre";
        hy.label = "hy";
        hy.titre = "";
        addParamOut(hy);
		
		// fct de rpartition
        ParamOut fx = new ParamOut();
        fx.type = "nombre";
        fx.label = "fx";
        fx.titre = "";
        addParamOut(fx);
        ParamOut fy = new ParamOut();
        fy.type = "nombre";
        fy.label = "fy";
        fy.titre = "";
        addParamOut(fy);
        
		// Gauss limit
        ParamOut gx = new ParamOut();
        gx.type = "nombre";
        gx.label = "gx";
        addParamOut(gx);
        ParamOut gy = new ParamOut();
        gy.type = "nombre";
        gy.label  = "gy";
        gy.titre = "";
        addParamOut(gy);
		
		// infos (n itrations)
        ParamOut info = new ParamOut();
        info.type = "string";
        info.label = "info";
        addParamOut(info);
		
        // options d'affichage
		appletDoc("Histogram_doc.xml");
        
        // affichage "plot"
        affPlot = new Affichage();
        affPlot.type = "plot";
        affPlot.titre = "bruit de Poisson - histogramme";
        String[][] pparams = {{"hx", "hy"},{"fx","fy"}};
        affPlot.params = pparams;
		affPlot.histogramme = true;
		affPlot.effacer = true;
		affPlot.recadrer = true;
        addAffichage(affPlot);

        // affichage "plotlive"
        affPlotLive = new Affichage();
        affPlotLive.type = "plotlive";
        affPlotLive.titre = "moyenne";
        String[][] pparams2 = {{"hx", "hy"},{"fx","fy"},{"gx","gy"}};
        affPlotLive.params = pparams2;
		affPlotLive.effacer = true;
		affPlotLive.connecter = true;
		affPlotLive.points = "none";
        addAffichage(affPlotLive);
		
        // affichage "tableau"
        Affichage affTab = new Affichage();
        affTab.type = "tableau";
        affTab.titre = "tableau";
        affTab.params = pparams;
        addAffichage(affTab);
    }
	
    // calcul complet d'un coup (utilis avec les affichages "plot" et "tableau")
    public ListeValeurs calcul(ListeValeurs in) throws SimuException {
        mu = in.lireDouble("mu");
        npts = in.lireEntier("npts");
		distrib = "on".equals(in.lireString("distrib"));
		
		int[] valeurs = Proba.sPoisson(mu, npts);
		int hmin = MoreProba.minval(valeurs);
		int hmax = MoreProba.maxval(valeurs);
		int nh = hmax - hmin + 1;
		double[] histo = MoreProba.doHistoSample(MoreProba.tableauDouble(valeurs), hmin, hmax, nh);
		
		double[] boxS = MoreProba.bestBoxS(mu, npts);
		double xminS = boxS[0];
		double xmaxS = boxS[1];
		double yminS = boxS[2];
		double ymaxS = boxS[3];
		double[] boxH = MoreProba.bestBoxH(mu, npts);
		double xminH = boxH[0];
		double xmaxH = boxH[1];
		double yminH = boxH[2];
		double ymaxH = boxH[3];
		
		double xmin = yminS - 0.5;
		double xmax = ymaxS;
		double ymin = yminH;
		double ymax = ymaxH;
		
		affPlot.xdebut = "" + xmin;
		affPlot.xfin = "" + xmax;
		affPlot.ydebut = "" + ymin;
		affPlot.yfin = "" + ymax;
		
        ListeValeurs out = new ListeValeurs();
		for (int i=0; i<nh; i++) {
			out.ajouterDouble("hx", hmin + i);
			out.ajouterDouble("hy", histo[i]);
		}
		
		if (distrib) {
			double proba;
			for (int k=(int)Math.floor(xmin); k<Math.ceil(xmax)+1; k++)
				if (k >= 0) {
					try {
						proba = Proba.pPoisson(mu,k)*npts;
						out.ajouterDouble("fx", k+0.2);
						out.ajouterDouble("fy", proba);
					} catch (Exception ex) {
					}
				}
		} else { // pb affichage: comment ne pas avoir  mettre de valeur ici ?
			out.ajouterDouble("fx", 0);
			out.ajouterDouble("fy", 0);
			out.ajouterDouble("fx", 0);
			out.ajouterDouble("fy", 0);
		}
		
		out.ajouterString("info", "");
		
        return out;
    }
	
    // calcul live (utilis avec affichage "plotlive")
    // initialisation
    public void initCalculLive(ListeValeurs in) throws SimuException {
        mu = in.lireDouble("mu");
        npts = in.lireEntier("npts");
		distrib = "on".equals(in.lireString("distrib"));
		tcl = "on".equals(in.lireString("tcl"));
		
		double[] boxH = MoreProba.bestBoxH(mu, npts);
		double xmin = boxH[0];
		double xmax = boxH[1];
		double ymin = boxH[2];
		double ymax = boxH[3];
		
		affPlotLive.xdebut = "" + xmin;
		affPlotLive.xfin = "" + xmax;
		affPlotLive.ydebut = "" + ymin;
		affPlotLive.yfin = "" + ymax;
		
		if (tcl) {
			double Dx = (xmax-xmin)/(double)(nGauss-1);
			tx = new double[nGauss];
			for (int i=0; i<nGauss; i++)
				tx[i] = xmin + Dx*i;
			gauss = new double[nGauss];
			for (int i=0; i<nGauss; i++)
				gauss[i] = npts * Math.exp(-0.5*Math.pow(tx[i] - mu, 2)/mu) / Math.sqrt(2.0*Math.PI*mu);
		}
		
		hmin = Math.round(xmin);
		hmax = Math.round(xmax);
		nh = (int)(hmax-hmin) + 1;
		
		histoM = new double[nh];
		for (int i=0; i<nh; i++)
			histoM[i] = 0;
		
		if (distrib) {
			// Law of great numbers [traduction = la loi des nombres clbres ?]
			tfx = new double[(int)Math.ceil(xmax) - (int)Math.floor(xmin)];
			tfy = new double[(int)Math.ceil(xmax) - (int)Math.floor(xmin)];
			int i=0;
			for (int k=(int)Math.floor(xmin); k<Math.ceil(xmax)+1; k++)
				if (k >= 0) {
					try {
						tfx[i] = k;
						tfy[i] = Proba.pPoisson(mu,k)*npts;
						i++;
					} catch (Exception ex) {
					}
				}
		}
		
		nt = 0;
		ct = System.currentTimeMillis();
    }
    
    // Calcul du temps t au temps t+1.
    public ListeValeurs calculLive() throws SimuException {
        if (nt >= NTIMES)
            return null; // fin du calcul
		
		for (int ii=0; ii<nipa; ii++) {
			nt++;
			int[] valeurs = Proba.sPoisson(mu, npts);
			
			double[] histo = MoreProba.doHistoSample(MoreProba.tableauDouble(valeurs), hmin, hmax, nh);
			
			for (int i=0; i<nh; i++)
				histoM[i] = ((nt-1)*histoM[i] + histo[i])/nt;
		}
        ListeValeurs out = new ListeValeurs();
		for (int i=0; i<nh; i++) {
			out.ajouterDouble("hx", hmin + i-0.5);
			out.ajouterDouble("hy", histoM[i]);
			
			out.ajouterDouble("hx", hmin + i+0.5);
			out.ajouterDouble("hy", histoM[i]);
		}
		
		if (distrib) {
			for (int i=0; i<tfx.length; i++) {
				out.ajouterDouble("fx", tfx[i]-0.5);
				out.ajouterDouble("fy", tfy[i]);
				
				out.ajouterDouble("fx", tfx[i]+0.5);
				out.ajouterDouble("fy", tfy[i]);
			}
		} else { // pb affichage: comment ne pas avoir  mettre de valeur ici ?
			out.ajouterDouble("fx", 0);
			out.ajouterDouble("fy", 0);
			out.ajouterDouble("fx", 0);
			out.ajouterDouble("fy", 0);
		}
		if (tcl) {
			for (int i=0; i<gauss.length; i++) {
				out.ajouterDouble("gx", tx[i]);
				out.ajouterDouble("gy", gauss[i]);
			}
		} else { // pb affichage: comment ne pas avoir  mettre de valeur ici ?
			out.ajouterDouble("gx", 0);
			out.ajouterDouble("gy", 0);
			out.ajouterDouble("gx", 0);
			out.ajouterDouble("gy", 0);
		}
		
		out.ajouterString("info", "nb d'itrations: " + nt);
		
		// pour bloquer le programme pendant dt-tempo secondes
		long ct2 = System.currentTimeMillis();
		if (ct2 - ct < tempo) {
	        try {
	            Thread.currentThread().sleep(tempo - (ct2 - ct));
	        } catch (InterruptedException e) {}
			ct += tempo;
		} else
			ct = ct2;
		
        return out;
    }
}
