/*
PFEdemo1 - Probas

Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import simu.*;

//utilise Proba et MoreProba

/**
 * Algorithme de simulation pour SimuLab.
 * TCL
 */

//	compilation: javac -target 1.1 -bootclasspath ~/Java/jdk1.1/JDKClasses.zip -extdirs "" -encoding ISO-8859-1 -classpath .:SimuLab.jar TCL.java

public class TCL extends SimuApplet {
	final static int NTIMES = 1000;
	final static int tempo = 500; // un affichage / 0.5s
	final static int nipa = 1; // nombre d'itrations par affichage
	final static int nGauss = 128;
	final static double mu0 = 0.1;
	final static int mumax = 100;
	final static double fact = 3.5;
	Affichage affPlotLive;
	double xmin, xmax;
	long ct;
	int nt;
	double[] txn;
	double[] tphi;

    public TCL() {
        // paramtres d'entre
		
        // paramtres de sortie
		
		// sample
        ParamOut sx = new ParamOut();
        sx.type = "nombre";
        sx.label = "sx";
        sx.titre = "";
        addParamOut(sx);
        ParamOut sy = new ParamOut();
        sy.type = "nombre";
        sy.label = "sy";
        sy.titre = "";
        addParamOut(sy);
		
		// Gauss limit
        ParamOut gx = new ParamOut();
        gx.type = "nombre";
        gx.label = "gx";
        addParamOut(gx);
        ParamOut gy = new ParamOut();
        gy.type = "nombre";
        gy.label  = "gy";
        gy.titre = "";
        addParamOut(gy);
        
		// infos (n itrations)
        ParamOut info = new ParamOut();
        info.type = "string";
        info.label = "info";
        addParamOut(info);
		
        // options d'affichage
        
        // affichage "plotlive"
        affPlotLive = new Affichage();
        affPlotLive.type = "plotlive";
        affPlotLive.titre = "TCL";
        String[][] pparams = {{"sx", "sy"},{"gx","gy"}};
        affPlotLive.params = pparams;
		affPlotLive.points = "none";
		affPlotLive.connecter = true;
		affPlotLive.effacer = true;
        addAffichage(affPlotLive);
    }
	
    // calcul live (utilis avec affichage "plotlive")
    // initialisation
    public void initCalculLive(ListeValeurs in) throws SimuException {
		xmin = -fact;
		xmax = fact;
		double ymin = 0.0;
		double ymax = 1.01;
		
		affPlotLive.xdebut = "" + xmin;
		affPlotLive.xfin = "" + xmax;
		affPlotLive.ydebut = "" + ymin;
		affPlotLive.yfin = "" + ymax;
		
		// Compute reduced normal f.r.
		txn = new double[nGauss];
		tphi = new double[nGauss];
		double dx = (xmax - xmin) / (nGauss - 1.0);
		
		for (int i=0; i<nGauss; i++)
			txn[i] = xmin + dx*i;
		tphi = Proba.fnormal(0.0, 1.0, txn);
		
		nt = 0;
		ct = System.currentTimeMillis();
    }
    
    // Calcul du temps t au temps t+1.
    public ListeValeurs calculLive() throws SimuException {
        if (nt >= NTIMES)
            return null; // fin du calcul
		
		double[] tx = new double[1];
		double[] fx = new double[1];
		int nx = 1;
		for (int ii=0; ii<nipa; ii++) {
			nt++;
			double mu = mu0*nt;
			if (mu > mumax)
				return(null);
			// sample mean f.r.
			double xxmin = Math.max(0.0, Math.floor(mu - fact*Math.sqrt(mu)));
			double xxmax = Math.ceil(mu + fact*Math.sqrt(mu));
			nx = (int)Math.round(xxmax - xxmin) + 1;
			tx = new double[nx];
			fx = new double[nx];
			for (int i=0; i<nx; i++)
				tx[i] = xxmin + i;
			fx = Proba.fPoisson(mu, tx);
			for (int i=0; i<nx; i++)
				tx[i] = (tx[i]-mu)/Math.sqrt(mu);
		}
        ListeValeurs out = new ListeValeurs();
		out.ajouterDouble("sx", xmin);
		out.ajouterDouble("sy", 0);
		out.ajouterDouble("sx", tx[0]);
		out.ajouterDouble("sy", 0);
		for (int i=0; i<nx; i++) {
			if (i == nx - 1 || tx[i+1] > tx[i]) {
				out.ajouterDouble("sx", tx[i]);
				out.ajouterDouble("sy", fx[i]);
			}
			
			if (i != nx - 1 && tx[i+1] > tx[i]) {
				out.ajouterDouble("sx", tx[i+1]);
				out.ajouterDouble("sy", fx[i]);
			}
		}
		out.ajouterDouble("sx", xmax);
		out.ajouterDouble("sy", fx[nx-1]);
		
		for (int i=0; i<txn.length; i++) {
			out.ajouterDouble("gx", txn[i]);
			out.ajouterDouble("gy", tphi[i]);
		}
		
		out.ajouterString("info", "nb d'itrations: " + nt);
		
		// pour bloquer le programme pendant dt-tempo secondes
		long ct2 = System.currentTimeMillis();
		if (ct2 - ct < tempo) {
	        try {
	            Thread.currentThread().sleep(tempo - (ct2 - ct));
	        } catch (InterruptedException e) {}
			ct += tempo;
		} else
			ct = ct2;
		
        return out;
    }
}
