/*
Histogram - algorithme de simulation pour SimuLab

Copyright (C) 2012 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

//utilise Proba et MoreProba

new Histogram();

function Histogram() {
	this.mu = 0.;
	this.npts = 0;
	// variable/constantes pour l'affichage "plotlive":
	this.NTIMES = 400;
	this.nGauss = 128;
	this.nipa = 1; // nombre d'itérations par affichage
	this.nt = 0;
	this.histoM = null; // double[]
	this.hmin = 0.;
    this.hmax = 0.;
	this.nh = 0;
	this.distrib = true;
	this.tfx = null; // double[]
	this.tfy = null; // double[]
    this.tcl = true;
	this.tx = null; // double[]
	this.gauss = null; // double[]
    
    this.applet = new SimuApplet(this);
    this.applet.appletDoc('histogram_doc.xml');
    this.applet.setDelaiIterations(500); // un affichage / 0.5s
}

Histogram.prototype.calcul = function(lin) {
    var valeurs, hmin, hmax, nh, histo,
        boxS, xminS, xmaxS, yminS, ymaxS,
        boxH, xminH, xmaxH, yminH, ymaxH,
        xmin, xmax, ymin, ymax,
        affPlot, out, i, proba, k;
    
    this.mu = lin.lireDouble('mu');
    this.npts = lin.lireEntier('npts');
    this.distrib = (lin.lireString('distrib') == 'on');
    
    valeurs = Proba.sPoisson(this.mu, this.npts);
    hmin = MoreProba.minval(valeurs);
    hmax = MoreProba.maxval(valeurs);
    nh = hmax - hmin + 1;
    histo = MoreProba.doHistoSample(valeurs, hmin, hmax, nh);
    
    boxS = MoreProba.bestBoxS(this.mu, this.npts);
    xminS = boxS[0];
    xmaxS = boxS[1];
    yminS = boxS[2];
    ymaxS = boxS[3];
    boxH = MoreProba.bestBoxH(this.mu, this.npts);
    xminH = boxH[0];
    xmaxH = boxH[1];
    yminH = boxH[2];
    ymaxH = boxH[3];
    
    xmin = yminS - 0.5;
    xmax = ymaxS;
    ymin = yminH;
    ymax = ymaxH;
    
    affPlot = this.applet.getAffichage('plot');
    affPlot.xdebut = '' + xmin;
    affPlot.xfin = '' + xmax;
    affPlot.ydebut = '' + ymin;
    affPlot.yfin = '' + ymax;
    
    out = new ListeValeurs();
    for (i=0; i<nh; i++) {
        out.ajouterDouble('hx', hmin + i);
        out.ajouterDouble('hy', histo[i]);
    }
    
    if (this.distrib) {
        for (k=Math.floor(xmin); k<Math.ceil(xmax)+1; k++)
            if (k >= 0) {
                //try {
                    proba = Proba.pPoisson(this.mu, k) * this.npts;
                    out.ajouterDouble('fx', k);
                    out.ajouterDouble('fy', proba);
                //} catch (ex) {
                //}
            }
    } else { // pb affichage: comment ne pas avoir à mettre de valeur ici ?
        out.ajouterDouble('fx', 0);
        out.ajouterDouble('fy', 0);
        out.ajouterDouble('fx', 0);
        out.ajouterDouble('fy', 0);
    }
    
    out.ajouterString('info', '');
    
    return out;
}

Histogram.prototype.initCalculLive = function(lin) {
    var boxH, xmin, xmax, ymin, ymax, affPlotLive,
        Dx, i, k;
    
    this.mu = lin.lireDouble('mu');
    this.npts = lin.lireEntier('npts');
    this.distrib = (lin.lireString('distrib') == 'on');
    this.tcl = (lin.lireString('tcl') == 'on');
    
    boxH = MoreProba.bestBoxH(this.mu, this.npts);
    xmin = boxH[0];
    xmax = boxH[1];
    ymin = boxH[2];
    ymax = boxH[3];
    
    affPlotLive = this.applet.getAffichage('plotlive');
    affPlotLive.xdebut = '' + xmin;
    affPlotLive.xfin = '' + xmax;
    affPlotLive.ydebut = '' + ymin;
    affPlotLive.yfin = '' + ymax;
    
    if (this.tcl) {
        Dx = (xmax - xmin) / (this.nGauss - 1);
        this.tx = new Array(this.nGauss);
        for (i=0; i<this.nGauss; i++)
            this.tx[i] = xmin + Dx*i;
        this.gauss = new Array(this.nGauss);
        for (i=0; i<this.nGauss; i++)
            this.gauss[i] = this.npts * Math.exp(-0.5*Math.pow(this.tx[i] - this.mu, 2)/this.mu) / Math.sqrt(2.0*Math.PI*this.mu);
    }
    
    this.hmin = Math.round(xmin);
    this.hmax = Math.round(xmax);
    this.nh = Math.floor(this.hmax - this.hmin) + 1;
    
    this.histoM = new Array(this.nh);
    for (i=0; i<this.nh; i++)
        this.histoM[i] = 0;
    
    if (this.distrib) {
        // Law of great numbers
        this.tfx = new Array(Math.ceil(xmax) - Math.floor(xmin));
        this.tfy = new Array(Math.ceil(xmax) - Math.floor(xmin));
        i = 0;
        for (k=Math.floor(xmin); k<Math.ceil(xmax)+1; k++)
            if (k >= 0) {
                try {
                    this.tfx[i] = k;
                    this.tfy[i] = Proba.pPoisson(this.mu,k)*this.npts;
                    i++;
                } catch (ex) {
                }
            }
    }
    
    this.nt = 0;
}

Histogram.prototype.calculLive = function() {
    var ii, valeurs, histo, i, out;
    
    if (this.nt >= this.NTIMES)
        return null; // fin du calcul
    
    for (ii=0; ii<this.nipa; ii++) {
        this.nt++;
        valeurs = Proba.sPoisson(this.mu, this.npts);
        
        histo = MoreProba.doHistoSample(valeurs, this.hmin, this.hmax, this.nh);
        
        for (i=0; i<this.nh; i++)
            this.histoM[i] = ((this.nt-1)*this.histoM[i] + histo[i])/this.nt;
    }
    
    out = new ListeValeurs();
    for (i=0; i<this.nh; i++) {
        out.ajouterDouble('hx', this.hmin + i-0.5);
        out.ajouterDouble('hy', this.histoM[i]);
        
        out.ajouterDouble('hx', this.hmin + i+0.5);
        out.ajouterDouble('hy', this.histoM[i]);
    }
    
    if (this.distrib) {
        for (i=0; i<this.tfx.length; i++) {
            out.ajouterDouble('fx', this.tfx[i]-0.5);
            out.ajouterDouble('fy', this.tfy[i]);
            out.ajouterDouble('fx', this.tfx[i]+0.5);
            out.ajouterDouble('fy', this.tfy[i]);
        }
    } else { // pb affichage: comment ne pas avoir à mettre de valeur ici ?
        out.ajouterDouble('fx', 0);
        out.ajouterDouble('fy', 0);
        out.ajouterDouble('fx', 0);
        out.ajouterDouble('fy', 0);
    }
    if (this.tcl) {
        for (i=0; i<this.gauss.length; i++) {
            out.ajouterDouble('gx', this.tx[i]);
            out.ajouterDouble('gy', this.gauss[i]);
        }
    } else { // pb affichage: comment ne pas avoir à mettre de valeur ici ?
        out.ajouterDouble('gx', 0);
        out.ajouterDouble('gy', 0);
        out.ajouterDouble('gx', 0);
        out.ajouterDouble('gy', 0);
    }
    
    out.ajouterString('info', "nb d'itérations: " + this.nt);
    
    return out;
}
