/*
PoissonSample - algorithme de simulation pour SimuLab

Copyright (C) 2012 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

//utilise Proba et MoreProba

new PoissonSample();

function PoissonSample() {
	this.ntimes = 400;
    
	this.mu = 0.;
	this.npts = 0;
    
	this.nt = 0;
	this.tmean = null; // double[]
    
    this.applet = new SimuApplet(this);
    this.applet.appletDoc("poisson_sample_doc.xml");
    this.applet.setDelaiIterations(500); // un affichage / 0.5s
}

PoissonSample.prototype.calcul = function(lin) {
    var box, affPlot, valeurs, out, i;
    
    this.mu = lin.lireDouble('mu');
    this.npts = lin.lireEntier('npts');
    
    box = MoreProba.bestBoxS(this.mu, this.npts);
    affPlot = this.applet.getAffichage('plot');
    affPlot.xdebut = '' + box[0];
    affPlot.xfin = '' + box[1];
    affPlot.ydebut = '' + box[2];
    affPlot.yfin = '' + box[3];
    
    valeurs = Proba.sPoisson(this.mu, this.npts);
    out = new ListeValeurs();
    for (i=0; i<this.npts; i++) {
        out.ajouterDouble('sx', i + 1);
        out.ajouterDouble('sy', valeurs[i]);
    }
    
    out.ajouterString('info', '');
    
    return out;
}

PoissonSample.prototype.initCalculLive = function(lin) {
    var box, affPlot, i;
    this.mu = lin.lireDouble('mu');
    this.npts = lin.lireEntier('npts');
    
    box = MoreProba.bestBoxS(this.mu, this.npts);
    affPlot = this.applet.getAffichage('plotlive');
    affPlot.xdebut = '' + box[0];
    affPlot.xfin = '' + box[1];
    affPlot.ydebut = '' + box[2];
    affPlot.yfin = '' + box[3];
    
    this.tmean = new Array(this.npts);
    for (i=0; i<this.npts; i++)
        this.tmean[i] = 0;
    
    this.nt = 0;
}

PoissonSample.prototype.calculLive = function() {
    var valeurs, out, i;
    
    if (this.nt >= this.ntimes)
        return(null); // fin du calcul
    this.nt++;
    
    valeurs = Proba.sPoisson(this.mu, this.npts);
    out = new ListeValeurs();
    for (i=0; i<this.npts; i++) {
        out.ajouterDouble('sx', i+1);
        out.ajouterDouble('sy', valeurs[i]);
    }
    
    for (i=0; i<this.npts; i++) {
        out.ajouterDouble('mx', i + 1);
        this.tmean[i] = ((this.nt - 1) * this.tmean[i] + valeurs[i]) / this.nt;
        out.ajouterDouble('my', this.tmean[i]);
    }
    
    out.ajouterString('info', "nb d'itérations: " + this.nt);
    
    return out;
}
