/*
Proba

Copyright (C) 2012 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

/**
	fonctions définies dans Proba:
       pPoisson, fPoisson, sPoisson, &
       fnormal, pnormal, &
       fgamma, pgamma, &
       hpsort, ihpsort, iMerge1, rMerge1, locate,
	   gammp, gammq
 */

Proba = function() {

return {
    
    /**
     * mu: Moyenne de la loi de Poisson
     * k: valeur de la v.a.
     * throws Exception if mu < 0
     */
    //public static double pPoisson(double mu, int k) throws Exception {
    pPoisson: function(mu, k) {
        if (mu < 0)
            throw new Error("pPoisson: mu < 0");
        else if (k < 0 || mu == 0 )
           return(0);
        else
           return(Math.exp(k * Math.log(mu) - this.gamln(k + 1) - mu));
    },


    /**
     * Tres imprecis car on n'utilise pas la fonction Log de IGamma !!!
     */
    //public static double[] fPoisson(double mu, double[] x) {
    fPoisson: function(mu, x) {
        var xx, f2, f, i;
        
        f = new Array(x.length);
        for (i=0; i<x.length; i++) {
           xx = Math.floor(x[i]) + 1;
           if (xx > 0)
              f[i] = this.gammq(xx, mu);
           else
              f[i] = 0;
        }
        return(f);
    },


    /**
    * Generates a IID Poisson sample of size n with mean mu.
    */
    //public static int[] sPoisson(double mu, int n) {
    sPoisson: function(mu, n) {
        var t, dt, v, i, test;
        
        t = new Array(n);
        dt = new Array(n);
        v = new Array(n);

        for (i=0; i<n; i++) {
            v[i] = 0;
            t[i] = 0.;
        }
        test = true;
        for (i=0; i<n; i++)
            if (t[i] < mu)
                test = false;
        while (!test) {
           for (i=0; i<n; i++)
              dt[i] = Math.random();
           for (i=0; i<n; i++)
              t[i] = t[i] - Math.log(dt[i]);
           test = true;
            for (i=0; i<n; i++)
                if (t[i] < mu) {
                    test = false;
                    v[i] = v[i] + 1;
                }
        }
        return(v);
    },

    /**
    * Generates a IID Poisson sample of size 1 with mean mu.
    */
    //public static int sPoisson1(double mu) {
    sPoisson1: function(mu) {
        var t, v, dt, test;
        
        t = 0.;
        v = 0;
        test = (t < mu);
        while (test) {
           dt = Math.random();
           t -= Math.log(dt);
           test = (t < mu);
            if (test)
                v++;
        }
        return(v);
    },

    /**
      * Distribution function of a normal r.v.
      */
    //public static double[] fnormal(double mu, double sigma, double[] x) {
    fnormal: function(mu, sigma, x) {
        var n, y, f, i;
        
        n = x.length;
        y = new Array(n);
        f = new Array(n);

        for (i=0; i<n; i++)
            y[i] = (x[i] - mu) / sigma;
        for (i=0; i<n; i++)
           f[i] = this.ndf(y[i]);
         return(f);
    },

    /**
      * p.d.f. of a normal r.v.
      */
    //public static double[] pnormal(double mu, double sigma, double[] x) {
    pnormal: function(mu, sigma, x) {
        var n, y, p, h, i;
        
        n = x.length;
        y = new Array(n);
        p = new Array(n);
        
        for (i=0; i<n; i++)
            y[i] = (x[i] - mu) / sigma;
        h = Math.sqrt(-2.0*Math.log(Number.MIN_VALUE)); // h = Double.POSITIVE_INFINITY ?
        for (i=0; i<n; i++)
            if (Math.abs(y[i]) <= h)
                p[i] = Math.exp(-0.5 * y[i] * y[i]);
            else
                p[i] = 0;
        for (i=0; i<n; i++)
            p[i] = p[i] * (1.0 / Math.sqrt(2*Math.PI) / sigma);

         return(p);
    },


    /**
      * Distribution function of a Gamma (Erlang) r.v.
      */
    //public static double[] fgamma(double nu, double lambda, double[] x) {
    fgamma: function(nu, lambda, x) {
        var xx, n, f, i;
        
        n = x.length;
        f = new Array(n);

        for (i=0; i<n; i++) {
           xx = x[i]*lambda;
           if (xx > 0)
              f[i] = this.gammp(nu, xx);
           else
              f[i] = 0;
        }
        /*
        for (int i=0; i<n; i++)
            if (f[i] > 0)
                f[i] = Math.exp(Math.log(f[i])-gamln(nu));
        */
        return(f);
    },

    /**
    * entrée:
    *    nu: ordre de la loi ("Nombre de photons observes")
    *    lambda: taux d'arrivée des photons.
    *    x
    * sortie: p
    */
    //public static double[] pgamma(double nu, double lambda, double[] x) throws Exception {
    pgamma: function(nu, lambda, x) {
        var n, p, i;

        n = x.length;
        p = new Array(n);
        
        if (nu <= 0 || lambda <= 0)
            throw new Error("pgamma: nu <= 0 || lambda <= 0");
        else {
           if (nu == 1) {
              for (i=0; i<n; i++)
                if (x[i] < 0)
                    p[i] = 0;
                else
                    p[i] = lambda * Math.exp(-lambda * x[i]);
           } else {
              for (i=0; i<n; i++)
                if (x[i] < 0)
                    p[i] = 0;
                else
                    p[i] = Math.exp(nu * Math.log(lambda) - this.gamln(nu) - lambda*x[i]) * Math.pow(x[i], nu - 1);
           }
           return(p);
        }
    },


    /*
    Service routines
    */

    /**
      * Natural logarithm of the (complete) Gamma function.
      */
    //public static double gammln(double xx) {
    gammln: function(xx) {
        var stp, fpf, cof, x, tmp, ser;
        
        stp = 2.50662827465;
        fpf = 5.5;
        cof = [76.18009173, -86.50532033, 24.01409822, -1.231739516, .120858003E-2, -.536382E-5];
        
        x = xx - 1;
        tmp = x + fpf;
        tmp = (x + 0.5)*Math.log(tmp) - tmp;
        ser = 1;
        for (i=0; i<6; i++) {
           x = x + 1;
           ser = ser + cof[i]/x;
        }
        return(tmp + Math.log(stp * ser));
    },


    /**
      *     evaluation of LN(GAMMA(A)) for positive a
      *
      *     D = 0.5*(LN(2*PI) - 1)
      *
      * entrée: a
      * sortie: double
      *
      */

    //public static double gamln(double a) {
    gamln: function(a) {
        var t, w, n, d, c0, c1, c2, c3, c4, c5;
        
        d = .418938533204673;
        c0 = .833333333333333E-01;
        c1 = -.277777777760991E-02;
        c2 = .793650666825390E-03;
        c3 = -.595202931351870E-03;
        c4 = .837308034031215E-0;
        c5 = -.165322962780713E-02;
        
        if (a > 0.8) {
           if (a > 2.25) { 
              if (a >= 10.0) {
                 t = Math.pow(1/a, 2);
                 w = (((((c5*t + c4)*t + c3)*t + c2)*t + c1)*t + c0)/a;
                 return((d+w) + (a-0.5)* (Math.log(a)-1));
              } else {
                 n = Math.floor(a - 1.25);
                 t = a;
                 w = 1;
                 for (i=0; i<n; i++) {
                    t = t - 1;
                    w = t*w;
                 }
                 return(this.gamln1(t-1) + Math.log(w));
              }
           } else {
              t = (a-0.5) - 0.5;
              return(this.gamln1(t));
           }
        } else
           return(this.gamln1(a) - Math.log(a));
    },

    /**
    * evaluation of LN(GAMMA(1 + a)) for -0.2 <= a <= 1.25
    */
    //public static double gamln1(double a) {
    gamln1: function(a) {
        var w, x, p, q, r, s;
        
        p = [ .577215664901533, .844203922187225,
             -.168860593646662, -.780427615533591,
             -.402055799310489, -.673562214325671E-01,
             -.271935708322958E-02
        ];
        q = [ 0., /* q[0] inutilisé */
             .288743195473681E+01, .312755088914843E+01,
             .156875193295039E+01, .361951990101499E+00,
             .325038868253937E-01, .667465618796164E-03
        ];
        r = [.422784335098467E+00, .848044614534529E+00,
             .565221050691933E+00, .156513060486551E+00,
             .170502484022650E-01, .497958207639485E-03
        ];
        s = [ 0., /* s[0] inutilisé */
             .124313399877507E+01, .548042109832463E+00,
             .101552187439830E+00, .713309612391000E-02,
             .116165475989616E-03
        ];

        if (a >= 0.6) {
           x = (a - 0.5) - 0.5;
           w = (((((r[5]*x + r[4])*x + r[3])*x + r[2])*x + r[1])*x + r[0]) /
                (((((s[5]*x + s[4])*x + s[3])*x + s[2])*x + s[1])*x + 1);
           return(x * w);
           
        } else {
           w = ((((((p[6]*a + p[5])*a + p[4])*a + p[3])*a + p[2])*a + p[1])*a + p[0]) /
                ((((((q[6]*a + q[5])*a + q[4])*a + q[3])*a + q[2])*a + q[1])*a + 1);
           return(-a * w);
        }
    },

    /*
        Sort/Merge Programs
    */

    /**
      *	Indexes an array arr[0:n-1] i.e., outputs the array indx[0:n-1] such that 
      * arr[indx[j]] is in ascending order for j=0,1,...,n-1.
      * 
      * entrée: n, arr
      * sortie: indx
      */

    //public static void indexx(int n, double[] arr, int[] indx) {
    indexx: function(n, arr, indx) {
        var m, nstack, i, indxt, ir, itemp, k, l, j, jstack, istack, a;
        
        m = 7;
        nstack = 50;
        ir = n;
        l = 1;
        jstack = 0;
        istack = new Array(nstack + 1);
        
        for (j=1;j<=n;j++)
            indx[j] = j;
        for (;;) {
            if (ir-l < m) {
                for (j=l+1;j<=ir;j++) {
                    indxt = indx[j];
                    a = arr[indxt];
                    for (i=j-1; i>=1; i--) {
                        if (arr[indx[i]] <= a)
                            break;
                        indx[i+1] = indx[i];
                    }
                    indx[i+1] = indxt;
                }
                if (jstack == 0)
                    break;
                ir = istack[jstack--];
                l = istack[jstack--];
            } else {
                k = (l+ir) >> 1;
                itemp = indx[k];
                indx[k] = indx[l+1];
                indx[l+1] = itemp;
                if (arr[indx[l+1]] > arr[indx[ir]]) {
                    itemp = indx[l+1];
                    indx[l+1] = indx[ir];
                    indx[ir] = itemp;
                }
                if (arr[indx[l]] > arr[indx[ir]]) {
                    itemp = indx[l];
                    indx[l] = indx[ir];
                    indx[ir] = itemp;
                }
                if (arr[indx[l+1]] > arr[indx[l]]) {
                    itemp = indx[l+1];
                    indx[l+1] = indx[l];
                    indx[l] = itemp;
                }
                i = l+1;
                j = ir;
                indxt = indx[l];
                a = arr[indxt];
                for (;;) {
                    do i++; while (arr[indx[i]] < a);
                    do j--; while (arr[indx[j]] > a);
                    if (j < i)
                        break;
                    itemp = indx[i];
                    indx[i] = indx[j];
                    indx[j] = itemp;
                }
                indx[l] = indx[j];
                indx[j] = indxt;
                jstack += 2;
                if (jstack > nstack)
                    alert("NSTACK too small in indexx.");
                if (ir-i+1 >= j-l) {
                    istack[jstack] = ir;
                    istack[jstack-1] = i;
                    ir = j-1;
                } else {
                    istack[jstack] = j-1;
                    istack[jstack-1] = l;
                    l = i;
                }
            }
        }
    },

    /**
      *  Sort *number* array ra in ascending order
      * entrée-sortie: ra
      */
    //public static void hpsort(double[] ra) {
    hpsort: function(ra) {
        var i, ir, j, l, n, rra;
        
        n = ra.length;
        if (n < 2)
            return;
        l = (n >> 1);
        ir = n-1;
        for (;;) {
            if (l > 0) {
                rra = ra[--l];
            } else {
                rra = ra[ir];
                ra[ir] = ra[0];
                if (--ir == 0) {
                    ra[0] = rra;
                    break;
                }
            }
            i = l;
            j = l+l+1;
            while (j <= ir) {
                if (j < ir && ra[j] < ra[j+1])
                    j++;
                if (rra < ra[j]) {
                    ra[i] = ra[j];
                    i = j;
                    j = (j << 1) + 1;
                } else j = ir + 1;
            }
            ra[i] = rra;
        }
    },

    /**
    * Merge last element of otherwise sorted integer array a[0:na]
    * entrée-sortie: a
    */
    //public static void iMerge1(int[] a) {
    //public static void iMerge1(int[] a, int na) {
    iMerge1: function(a, na) {
        var inotsup, isave, i;
        
        if (na == undefined)
            na = a.length;
        
        if (na <= 1)
            return;
        
        inotsup = this.iLocate(a, na);

        // Do the right shift to make space for a[na-1]
        isave = a[na-1];
        for (i=na-1; i>=inotsup+2; i--)
            a[i] = a[i-1];
        a[inotsup+1] = isave;
    },

    /**
    * Merge last element of otherwise sorted double array a[0:na-1]
    * entrée-sortie: a
    */
    //public static void rMerge1(double[] a) {
    //public static void rMerge1(double[] a, int na) {
    rMerge1: function(a, na) {
        var inotsup, isave, i;
        
        if (na == undefined)
            na = a.length;
        
        if (na <= 1)
            return;
        
        inotsup = this.rLocate(a[na-1], a, na-1);

        // Do the right shift to make space for a[na-1]
        isave = a[na-1];
        for (i=na-1; i>=inotsup+2; i--)
            a[i] = a[i-1];
        a[inotsup+1] = isave;
    },


    /**
      * Locate returns rank of greatest a[0:n-1] not greater than x.
      * a[0:n-1] _must_ be sorted in ascending order.
      */
    //public static int rLocate(double x, double[] a, int na) {
    rLocate: function(x, a, na) {
        var imin, imax, itest;
        
        imin = -1;
        imax = na;
        for (;;) {
           itest = Math.round(0.5*(imin + imax));
           if (x > a[itest])
              imin = itest;
           else
              imax = itest;
           if (imax-imin <= 1)
                break;
        }
        return(Math.min(imin, imax));
    },


    /**
      * Locate returns rank of greatest a[0:n-2] not greater than a[n-1].
      * a[0:n-2] _must_ be sorted in ascending order.
      */
    //public static int iLocate(int[] a) {
    //public static int iLocate(int[] a, int n) {
    iLocate: function(a, n) {
        var x, na, imin, imax, itest;
        
        if (n == undefined)
            n = a.length;
        
        x = a[n-1];

        na = n-1;
        imin = -1;
        imax = na;
        for (;;) {
           itest = Math.round(0.5*(imin+imax));
           if (x > a[itest])
              imin = itest;
           else
              imax = itest;
           if (imax-imin <= 1)
                break;
        }
        return(Math.min(imin, imax));
    },

    /**
    * ndf returns the normal distribution function
    */
    //public static double ndf(double x) {
    ndf: function(x) {
        return(this.erf(x / Math.sqrt(2)) / 2 + 0.5);
    },

    /**
    * error function erf(x)
    */
    //public static double erf(double x) {
    erf: function(x) {
        if (x < 0)
          return(-this.gammp(0.5, Math.pow(x, 2)));
        else
          return(this.gammp(0.5, Math.pow(x, 2)));
    },

    /**
      * gammln pour gammp... semble différente de l'autre gammln... normalisée ?
      *
      * Returns the value ln[G(x)] for x > 0.
      * Internal arithmetic will be done in double precision, a nicety that you can omit if five-figure
      * accuracy is good enough.
      */
    //public static double gammln2(double x) {
    gammln2: function(x) {
        var cof, stp, fpf, ser, tmp, y, i;
        
        cof = [ 76.18009172947146e0, -86.50532032941677e0,
            24.01409824083091e0, -1.231739572450155e0,  .1208650973866179e-2,
            -.5395239384953e-5 ];
        stp = 2.5066282746310005e0;
        fpf = 5.5;
        
        y = x;
        tmp = x + fpf;
        tmp = (x + 0.5)*Math.log(tmp) - tmp;
        ser = 1.000000000190015;
        for (i=0; i<6; i++) {
            y = y + 1.;
            ser = ser + cof[i]/y;
        }
        return(tmp + Math.log(stp*ser/x));
    },

    /**
      * Returns the incomplete gamma function P(a; x).
      */
    //public static double gammp(double a, double x) {
    gammp: function(a, x) {
        if (x < 0 || a <= 0)
            alert("bad arguments in gammp");
        if (x < a + 1)
            return(this.gser(a, x));
        else
            return(1 - this.gcf(a, x));
    },

    /**
    * Returns the incomplete gamma function Q(a; x) = 1 - P(a; x).
    */
    //public static double gammq(double a, double x) {
    gammq: function(a, x) {
        if (x < 0 || a <= 0)
            alert("bad arguments in gammq");
        if (x < a + 1)
            return(1 - this.gser(a, x));
        else
            return(this.gcf(a, x));
    },

    /**
    * Returns the incomplete gamma function P(a; x) evaluated by its series representation
    */
    //public static double gser(double a, double x) {
    gser: function(a, x) {
        var itmax, eps, gln, ap, del, sum, problem, i;
        
        itmax = 100;
        eps = 3.e-7;
        gln = this.gammln2(a);
        if (x <= 0) {
            if (x < 0)
                alert("x < 0 in gser");
            return(0);
        }
        ap = a;
        sum = 1./a;
        del = sum;
        problem = true;
        for (i=0; i<itmax; i++) {
            ap = ap + 1.;
            del = del * x / ap;
            sum = sum + del;
            if (Math.abs(del) < Math.abs(sum)*eps) {
                problem = false;
                break;
            }
        }
        if (problem)
            alert("a too large, ITMAX too small in gser");
        return(sum * Math.exp(-x + a * Math.log(x) - gln));
    },

    /**
     * Returns the incomplete gamma function Q(a; x) evaluated by its continued fraction representation
     * as gammcf.
     * Parameters: ITMAX is the maximum allowed number of iterations; EPS is the relative accuracy;
     * FPMIN is a number near the smallest representable floating-point number.
     */
    //public static double gcf(double a, double x) {
    gcf: function(a, x) {
        var itmax, eps, fpmin, gln, an, b, c, d, del, h, problem, i;
        itmax = 100;
        eps = 3.e-7;
        fpmin = 1.e-30;
        gln = this.gammln2(a);
        b = x + 1. - a;
        c = 1./fpmin;
        d = 1./b;
        h = d;
        problem = true;
        for (i=1; i<itmax; i++) {
            an = -i * (i - a);
            b = b + 2.;
            d = an * d + b;
            if (Math.abs(d) < fpmin)
                d = fpmin;
            c = b + an / c;
            if (Math.abs(c) < fpmin)
                c = fpmin;
            d = 1. / d;
            del = d * c;
            h = h * del;
            if (Math.abs(del-1.) < eps) {
                problem = false;
                break;
            }
        }
        if (problem)
            alert("a too large, ITMAX too small in gcf");
        return(Math.exp(-x + a*Math.log(x) - gln) * h);
    }

};

}();
